#include "config.h"
#include "libwtref.h"
#include "exc.h"
#include <stdio.h>

wt_bool libwtref_run_vm(wt_profile vm, struct wtr_context* const ctx,
	LIBWTREF_PFN_ERROR_CB errcb, void* errcb_priv, wt_u32* const retc,
	wt_u32 retmax, wt_value* retv)
{
	wt_error err;

	for(;;)
	{
		enum wtr_runresult rr = wtrContextRun(ctx, 100000);

		switch(rr)
		{
		case WTRR_INIT:
			/* returned if the context is not set up -- this state */
			/* considered okay, since there is no error to report */
			return WT_TRUE;

		case WTRR_RETURNED:
			goto returned;

		case WTRR_RUN:
		case WTRR_QUANTUM:
		case WTRR_RAISE:
			/* nothing to do, just call wtrContextRun() again */
			break;

		case WTRR_EXCEPTION:
			{
				wt_exception exc;
				char excname[128];

				if(WTE_OK!=(err=wtrContextGetException(ctx, &exc)))
				{
					errcb(errcb_priv, "Could not get exception ID", err);
				}
				else
				if(WTE_OK!=(err=wtrProfileExceptionToString(vm, exc, excname,
					sizeof(excname)-1)))
				{
					errcb(errcb_priv, "Failed to get exception string", err);
				}
				else
				{
					errcb(errcb_priv, excname, WTE_URTEX);
				}

				return WT_FALSE;
			}
			break;

		case WTRR_IOCALL:
			libwtref_handle_iocall(ctx, errcb, errcb_priv);
			break;

		case WTRR_EXTCALL:
			/* should not reach normally, external calls should happen */
			/* internally, unless external function call is disabled (or */
			/* an external function yields) -- we do it here, because it */
			/* is not a big deal */
			{
				struct wtr_extcallinfo* callinfo;
				wt_value*               argv;

				if(WTE_OK!=(err=wtrContextGetExtCallParams(ctx, &callinfo,
					&argv)))
				{
					errcb(errcb_priv,
						"Failed to get external call parameters", err);
					return WT_FALSE;
				}

				callinfo->func(ctx, callinfo, argv);

				/* TODO: the only problem here is if the function is */
				/* broken, and always yields, then this become an infinite */
				/* loop -- how can we handle this? */
			}
			break;
		}
	}

returned:;
	wtrContextPopReturnValues(ctx, retc, retmax, retv);
	return WT_TRUE;
}
