/* $Id: ast_enumerator.h 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_ENUMERATOR_H
#define WTPARSER_AST_ENUMERATOR_H

#include <string>
#include "../astnode.h"
#include "../../wtruntime/wtruntime.h"

namespace wtast
{

class Type;
class EnumType;

class Enumerator : public AstNode
{
	std::string mName;
	wt_u32      mOrdinal;

public:
	Enumerator(const std::string & name)
		: AstNode()
		, mName(name)
		, mOrdinal(0)
	{
		// ok
	}

	~Enumerator();

	void Accept(AstVisitor* vis);

	Enumerator* Copy();

	Kind GetKind() const
	{
		return ENUMERATOR;
	}

	std::string & GetName()
	{
		return mName;
	}

	wt_u32 GetOrdinal() const
	{
		return mOrdinal;
	}

	friend class EnumType;

private:

	void SetOrdinal(wt_u32 ordinal)
	{
		mOrdinal = ordinal;
	}
};

} // namespace wtast

#endif // WTPARSER_AST_ENUMERATOR_H
