#ifndef WTRUNTIME_FUNCTION_H
#define WTRUNTIME_FUNCTION_H

#include "config.h"
#include "types.h"

struct wtr_profile;
struct wtr_program;
struct wtr_funcsig;
struct wtr_function;
struct wtr_cfunction;

struct wtr_function
{
	struct wtr_profile*   profile;        /* the object belong to this profile */
	struct wtr_program*   program;        /* the program this function belongs (only in WT function mode); NULL otherwise */
	struct wtr_cfunction* cfunc;          /* C function descriptor (only in C function mode); NULL otherwise */
	struct wtr_funcsig*   funcsig;        /* function signature */
	struct wtr_node       link_functions; /* link in the list of all functions */
	wt_pfn                pfn;            /* PFN structure used to execute this function */
	wt_uintptr            refcount;       /* number of references to this function */
	wt_uintptr            hancount;       /* number of handles created to this function */
};

struct wtr_cfunction
{
	struct wtr_profile*   profile;        /* the object belong to this profile */
	struct wtr_funcsig*   funcsig;        /* function signature */
	WT_PFN_EXTFUNC        extfunc_pfn;    /* function to call */
	void*                 token;          /* token supplied by the user */
};


/* Create a new function call descriptor. The ownership of 'sig' and 'cfunc' */
/* will be taken away (these will be freed up when it is released). */
/* Required arguments for C functions (or external functions): 'cfunc' */
/* ('prog' and 'pc' must be zero); required arguments for WT functions: */
/* 'prog' and 'pc' ('cfunc' must be NULL). */
struct wtr_function* wtr_function_new(struct wtr_profile* profile,
	struct wtr_funcsig* sig, struct wtr_cfunction* cfunc,
	struct wtr_program* prog, wt_u32 pc);
void wtr_function_release(struct wtr_profile* profile, struct wtr_function* func);

/* Create a handle to the given function. */
wt_function wtr_function_genhandle(struct wtr_profile* profile, struct wtr_function* func);
void wtr_function_remhandle(struct wtr_profile* profile, wt_function func);

/* Create a C function (or external function). */
struct wtr_cfunction* wtr_cfunction_new(struct wtr_profile* profile, struct wtr_funcsig* sig, WT_PFN_EXTFUNC func, void* token);
void wtr_cfunction_delete(struct wtr_profile* profile, struct wtr_cfunction* cfunc);

#endif /* WTRUNTIME_FUNCTION_H */
